#
#
# service_info.py
# (c) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
import blueman.Globals as Globals

import os
import bluetooth
import gtk
import threading
import gobject
from blueman.Main.marshaller import marshaller

#lock = threading.RLock()




class service_info():
	connections = []
	def __init__(self, address):
		self.address = address

		#lock.acquire()
		self.services = marshaller(os.path.expanduser("~/.blueman/services.dat"))
		
		if self.services.data == None:
			self.services.data = {}
			
		#lock.release()

	def _get_cached(self):
		try:
			
			info = self.services.data[self.address]

		except:
			
			info = None
		return info
		
		
	def _lock(self):
		Globals.dbg.info("service_info::_lock() [%s]" % self.address, 7)
		service_info.connections.append(self.address)
		
	def _unlock(self):
		Globals.dbg.info("service_info::_unlock() [%s]" % self.address, 7)
		service_info.connections.remove(self.address)
		
	def locked(self):
		return self.address in service_info.connections
				
	
	def find_service(self):
		return bluetooth.find_service(None, None, self.address)
		
					
	def _get(self, services, callback=None):
					
		handles = Globals.adapter.GetRemoteServiceHandles(self.address)
		
		for i in range(len(handles)):
			services[i]["handle"] = hex(handles[i])
			
		
		self.services.data[self.address] = services
		self.services.save()
		self._unlock()
		if callback != None:
			callback(services)
		else:
			return services
				
	def get(self, bypass_cache=False, allow_connection=True):
		if not bypass_cache:
			info = self._get_cached()
		else:
			info = None

		
		if not allow_connection:
			return info
			
		if not bypass_cache and info != None:
			return info
		else:
			self._lock()
			services = self.find_service()
			self._unlock()
			return _get(services)


	
	def get_async(self, callback, bypass_cache=False, allow_connection=True):
		Globals.dbg.info("service_info::get_async()", 7)
		if self.locked():
			Globals.dbg.info("service_info::locked(), Bailing", 7)
			#gobject.idle_add(callback, None)
			return
			
		if not bypass_cache:
			info = self._get_cached()
		else:
			info = None
		
		if not allow_connection:
			gobject.idle_add(callback, info)
		
		def thread(bypass_cache, allow_connection):
			Globals.dbg.info("service_info::get_async::thread()", 7)
			services = self.find_service()
			if len(services) == 0:
				gobject.idle_add(callback, info)
				gobject.idle_add(self._unlock)
			else:
			#gtk.gdk.threads_enter()
				gobject.idle_add(self._get, services, callback)
			#gtk.gdk.threads_leave()
		

			
		if not bypass_cache and info != None:
			gobject.idle_add(callback, info)
		else:
			self._lock()
			threading.Thread(target=thread, args=(bypass_cache, allow_connection,)).start()

		
		
	def has_service(self, uuid):
		try:
			services = self.services.data[self.address]

			for x in services:
				if uuid in x["service-classes"]:
					return True
				
			return False
		except:
			return False

