#
#
# context_menu.py
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
import gtk
import blueman.Globals as Globals
import bluetooth
_ =Globals._

NET_CONNECTING = 2
NET_ADDING = 4

AUDIO_ADDING = 8
AUDIO_CONNECTING = 16

INPUT_ADDING = 32
INPUT_CONNECTING = 64

from blueman.Bluetooth.service_info import service_info


class context_menu():
	instance = None
	def __init__(self, address, event):
		
		self.address = address
		self.xml = gtk.glade.XML(Globals.GLADE_DIR + "/context_menu.glade")
		self.menu = self.xml.get_widget("menu")
		self.menu.connect("selection-done", self.on_selection_done)
		
		refresh = self.xml.get_widget("refresh")
		refresh.connect("activate", self.on_refresh)
		
		self.network = self.xml.get_widget("network")
		
		self.sinfo = service_info(self.address)
		self.rmt_services = self.sinfo.get(False, False)
		

		state = False
	
		#self.init_services()
		
		status = self.init_serial()
		state = state or status
		
		status = self.init_audio_setup()
		state = state or status
		
		status = self.init_network_setup()
		state = state or status
		
		status = self.init_input_setup()
		state = state or status
		
		
		self.init_network_actions()
		#self.init_input_actions()
		self.init_audio_actions()
		
		
		if not state:
			sep = self.xml.get_widget("sep")
			sep.hide()

		
		context_menu.instance = self
		
		self.menu.popup(None, None, None, event.button, event.time)


	def on_refresh(self, widget):
		def callback(services):
			print services
		
		
		self.sinfo.get_async(callback, True)


	def init_services(self):
		#def callback(services):
		#	print services
		
		x=service_info(self.address)





	def set_state(self, state, info):
		try:
			if Globals.context_menu_states[self.address][0] != 0:
				if context_menu.instance != None:
					context_menu.instance.on_selection_done(None)
			Globals.context_menu_states[self.address] = (state, info)
		
		
		except AttributeError:
			Globals.context_menu_states = {}
			
		Globals.context_menu_states[self.address] = (state, info)
		
	def get_state(self):

		try:
			return Globals.context_menu_states[self.address]
		except AttributeError:
			Globals.context_menu_states = {}
			Globals.context_menu_states[self.address] = (0,0)
			return (0,0)
		except KeyError:
			Globals.context_menu_states[self.address] = (0,0)
			return (0,0)


	def set_menuitem(self, text, image):
		item = gtk.ImageMenuItem()
		item.set_image(image)
		
		label = gtk.Label()
		label.set_text(text)
		label.set_alignment(0,0.5)

		label.show()
		
		item.add(label)
		
		return item
		
	def init_input_setup(self):
		service = Globals.Services.get("input")
		
		def on_add(widget):
			def reply(*args):
				Globals.Statusbar.push(_("Success"))
				d = gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_INFO, buttons=gtk.BUTTONS_OK, message_format="Device configured")
				d.format_secondary_markup("Now move your mouse or press some buttons on your keyboard")
				d.show()
				d.connect("response", lambda x, y:d.destroy())
				
				self.set_state(0, 0)
				
			def err(msg):
				Globals.Statusbar.push(_("Failed (%s)") % str(msg).split(": ")[1])
				self.set_state(0, 0)
			
			Globals.Statusbar.push(_("Adding Input device..."))
			self.set_state(INPUT_ADDING, None)
			service.create_device(self.address, reply_handler=reply, error_handler=err)
			
		
			
		
		dev_class = Globals.adapter.GetRemoteMajorClass(self.address)
		cond = False
		if self.rmt_services:
			for x in self.rmt_services:
			
				cond = cond or bluetooth.HID_CLASS.upper() in x["service-classes"]

				#print cond, x["name"], x["service-classes"]
			
		
		if dev_class == "peripheral" or cond:
			image = gtk.image_new_from_icon_name("mouse", 4)
			item = self.set_menuitem(_("Setup Input device"), image)
			item.connect("activate", on_add)
			item.show()

			self.menu.prepend(item)
			
			if not service.is_running():
				item.set_sensitive(False)
			

			return True
			
			
		else:
			return False

		
		
		

	def init_audio_setup(self):
		service = Globals.Services.get("audio")
		def on_add(widget):
			def reply(*args):
				Globals.Statusbar.push(_("Success"))
				self.set_state(0, 0)
				
			def err(msg):
				Globals.Statusbar.push(_("Failed (%s)") % str(msg).split(": ")[1])
				self.set_state(0, 0)
			
			Globals.Statusbar.push(_("Adding Audio device..."))
			self.set_state(AUDIO_ADDING, None)
			service.create_device(self.address, reply_handler=reply, error_handler=err)
			
		
			
		
		dev_class = Globals.adapter.GetRemoteMajorClass(self.address)
		cond = False
		if self.rmt_services:
			for x in self.rmt_services:
			
				cond = cond or bluetooth.ADVANCED_AUDIO_CLASS.upper() in x["service-classes"]
				cond = cond or bluetooth.AUDIO_SINK_CLASS.upper() in x["service-classes"]
				cond = cond or bluetooth.HANDSFREE_CLASS.upper() in x["service-classes"]
				cond = cond or bluetooth.HEADSET_CLASS.upper() in x["service-classes"]
				#print cond, x["name"], x["service-classes"]
			
		
		if dev_class == "audio/video" or cond:
			image = gtk.image_new_from_icon_name("audio-card", 4)
			item = self.set_menuitem(_("Setup Audio device"), image)
			item.connect("activate", on_add)
			item.show()

			self.menu.prepend(item)
			
			if not service.is_running():
				item.set_sensitive(False)
			

			return True
			
			
		else:
			return False
			
	
	def init_audio_actions(self):
		service = Globals.Services.get("audio")
		
		def on_connect(widget, info):
			def reply(*args):
				print "reply"
				
			def err(msg):
				print msg
				
			info[0].Connect(reply_handler=reply, error_handler=err)
			
		def on_disconnect(widget, info):
			print "disc"
		
		
		
		if Globals.Services.get_service_status("audio") == 1:
			dev = service.get_device(self.address)
			if dev:
				profiles = service.get_profiles(dev.object_path)
				if len(profiles):
					item = gtk.SeparatorMenuItem()
					item.show()
					self.menu.prepend(item)
					found = False
					found2 = False
					
					for x in profiles:
						connectable = service.is_connectable(x[0].object_path, x[1])
						connected = service.is_connected(x[0].object_path, x[1])
						
						if connectable and not connected:
							if not found:
								image = gtk.image_new_from_icon_name("audio-card", 4)
								item = self.set_menuitem(_("Connect to Audio service"), image)
								
								item.show()
								
								menu = gtk.Menu()
								menu.show()
								item.set_submenu(menu)
								
								self.menu.prepend(item)
								found = True
								
							image = gtk.image_new_from_icon_name("audio-card", 4)
							item = self.set_menuitem("%s" % x[2], image)
							item.connect("activate", on_connect, x)
							item.show()
							menu.append(item)
						
						
						
						if connected:
							if not found2:
								image = gtk.image_new_from_icon_name("disconnect", 4)
								item = self.set_menuitem(_("Disonnect Audio device"), image)
								item.connect("activate", on_disconnect, x)
								item.show()
								
								menu2 = gtk.Menu()
								menu2.show()
								item.set_submenu(menu2)
								
								self.menu.prepend(item)
								found2 = True
								
							image = gtk.image_new_from_icon_name("disconnect", 4)
							item = self.set_menuitem("%s" % x[2], image)
							item.show()
							menu2.append(item)
					
				
				





	def init_serial(self):
		service = Globals.Services.get("serial")
		
		def reply(path):
			port = service.get_port(path)
			Globals.Statusbar.push(_("Serial port added as %s") % port.GetDevice())
			
		def err(*args):
			Globals.Statusbar.push(_("Failed"))
			print args
		
		
		def on_add_port(widget, uuid):
			Globals.Statusbar.push(_("Adding serial port"))
			path = service.create_port(self.address, uuid, reply_handler=reply, error_handler=err)

		
		found = False
		menu = None
		
		
		if self.rmt_services:
			for x in self.rmt_services:
				if x["protocol"] == "RFCOMM":
					if not found:
						
						image = gtk.image_new_from_icon_name("modem", 4)
						item = self.set_menuitem(_("Add Serial port"), image)
						item.show()
						menu = gtk.Menu()
						menu.show()
						item.set_submenu(menu)
						self.menu.prepend(item)
						found = True
						if not service.is_running():
							item.set_sensitive(False)
							item.set_tooltip_text(_("Serial service is disabled"))
							return True
					
					#if len(x["profiles"]) > 0:
					image = gtk.image_new_from_icon_name("modem", 4)
					item = self.set_menuitem(_("%s (Channel %s)") % (x["name"], x["port"]), image)

					
					
					item.connect("activate", on_add_port, str(x["port"]))
					item.show()
					menu.append(item)
					
		return found


	def mkuuid128(self, uuid):
		return "0000%s-0000-1000-8000-00805F9B34FB" % uuid



	def init_network_actions(self):
		service = Globals.Services.get("network")
		
		def on_disconnect(widget, conn):
			conn.Disconnect()
			Globals.Statusbar.push(_("Disconnected"))
							
		def on_connect(widget, service, conn):
			def reply(*args):
				Globals.Statusbar.push(_("Connected"))
				self.set_state(0,0)

		
			def err(err):
				if self.get_state()[0] != 0:
					Globals.Statusbar.push(_("Failed (%s)" % str(err).split(": ")[1]))
					self.set_state(0,0)

		
		
			conn.Connect(reply_handler=reply, error_handler=err)
		
			name = conn.GetName()
		
			self.set_state(NET_CONNECTING, conn)

			Globals.Statusbar.push(_("Connecting to %s" % name))
		

			
		def on_cancel(widget, service, conn):
			conn.CancelConnect()
			self.set_state(0,0)
			Globals.Statusbar.push(_("Cancelled"))
	
		
		def on_remove(widget, service, conn):
			service.remove_connection(conn.object_path)
			Globals.Statusbar.push(_("Removed"))
		
		
		if self.get_state()[0] == NET_CONNECTING:
			item = gtk.SeparatorMenuItem()
			item.show()
			self.menu.prepend(item)
		
			image = gtk.Image()
			image.set_from_stock(gtk.STOCK_STOP, 4)
			item = self.set_menuitem(_("Cancel Connecting"), image)
			item.connect("activate", on_cancel, service, self.get_state()[1])

			item.show()
			self.menu.prepend(item)
			
		elif self.get_state()[0] == NET_ADDING:
			item = gtk.SeparatorMenuItem()
			item.show()
			self.menu.prepend(item)
		
			image = gtk.Image()
			image.set_from_stock(gtk.STOCK_NEW, 4)
			item = self.set_menuitem(_("Adding connection"), image)
			item.set_sensitive(False)

			item.show()
			self.menu.prepend(item)
		
		
		elif Globals.Services.get_service_status("network") == 1:
			conn = service.find_connection(self.address)
			if conn:
				if not conn.IsConnected():
					item = gtk.SeparatorMenuItem()
					item.show()
					self.menu.prepend(item)
					
					
					image = gtk.Image()
					image.set_from_stock(gtk.STOCK_DELETE, 4)
					item = self.set_menuitem(_("Remove %s") % conn.GetName(), image)
					item.connect("activate", on_remove, service, conn)

					self.menu.prepend(item)
					item.show()
					
					
					image = gtk.Image()
					image.set_from_stock(gtk.STOCK_CONNECT, 4)
					item = self.set_menuitem(_("Connect to %s") % conn.GetName(), image)
					item.connect("activate", on_connect, service, conn)
					

					self.menu.prepend(item)
					item.show()
				

					

				else:
					item = gtk.SeparatorMenuItem()
					item.show()
					self.menu.prepend(item)
					
					image = gtk.Image()
					image.set_from_stock(gtk.STOCK_DISCONNECT, 4)
					item = self.set_menuitem(_("Disconnect from %s") % conn.GetName(), image)
					item.connect("activate", on_disconnect, conn)
					item.show()
					self.menu.prepend(item)

	
	def init_network_setup(self):
		service = Globals.Services.get("network")
		

		################# Funcs #################
		
		def on_connection_create(widget, server):
			def reply(*args):
				Globals.Statusbar.push(_("%s is set up") % server)
				self.set_state(0, 0)

			
			def err(msg):
				self.set_state(0, 0)
				Globals.Statusbar.push(_("Failed (%s)") % str(msg).split(": ")[1])

		
			service = Globals.Services.get("network")
			service.create_connection(self.address, server, reply_handler=reply, error_handler=err)
			Globals.Statusbar.push(_("Setting up %s...") % server)
			self.set_state(NET_ADDING, service)
		
		

			
			
		################# End of funcs #################
		
		has_nap = self.sinfo.has_service(bluetooth.NAP_CLASS)
		has_gn = self.sinfo.has_service(bluetooth.GN_CLASS)
		
		if has_gn or has_nap:
			image = gtk.image_new_from_icon_name("network", 4)
			network_item = self.set_menuitem(_("Setup Network"), image)
			network_item.show()
			menu = gtk.Menu()
			menu.show()
			network_item.set_submenu(menu)
			self.menu.prepend(network_item)
			
		else:
			return False
		

		if self.sinfo.has_service(bluetooth.NAP_CLASS):
			image = gtk.image_new_from_icon_name("network", 4)
			item = self.set_menuitem(_("Network Access Point"), image)
			item.show()
			item.connect("activate", on_connection_create, "NAP")
			menu.prepend(item)
			
		if self.sinfo.has_service(bluetooth.GN_CLASS):
			image = gtk.image_new_from_icon_name("network", 4)
			item = self.set_menuitem(_("Group (ad-hoc) Network"), image)
			item.show()
			item.connect("activate", on_connection_create, "GN")

			menu.prepend(item)

					
		if not Globals.Services.get_service_status("network"):
			network_item.set_sensitive(False)
			network_item.set_tooltip_text(_("Network service is disabled"))
			

		return True
	
		
		
		
		
	def on_selection_done(self, menu):
		print "menu destroyed"
		self.menu.destroy()
		context_menu.instance = None
	
	def __del__(self):
		
		print "deleting ", self

		
	
			
