#
#
# network.py
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import blueman.Globals as Globals
import gobject
import dbus
import os
from blueman.Services.bluetooth.generic import generic as generic_service

from blueman.Gui.notification import notification

from blueman.Main.marshaller import marshaller

from blueman.Main.polkit_auth import polkit_auth
from blueman.Main.mechanism import mechanism

_ = Globals._

class network(generic_service):
	__gtype_name__ = 'network_service'
	def __init__(self):
		generic_service.__init__(self, "network")
		
		gobject.signal_new("connect", network, gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, (gobject.TYPE_STRING,))
		gobject.signal_new("disconnect", network, gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, (gobject.TYPE_STRING,))
		gobject.signal_new("connection-created", network, gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, (gobject.TYPE_STRING,))
		gobject.signal_new("connection-removed", network, gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, (gobject.TYPE_STRING,))
		gobject.signal_new("enabled", network, gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, (gobject.TYPE_STRING,))
		gobject.signal_new("disabled", network, gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, (gobject.TYPE_STRING,))
		
		gobject.signal_new("dhcp-begin-request", network, gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, (gobject.TYPE_STRING, gobject.TYPE_STRING,))
		gobject.signal_new("dhcp-end-request", network, gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, (gobject.TYPE_STRING, gobject.TYPE_INT, gobject.TYPE_STRING,))
		
		if Globals.Settings.net_nap_autostart and self.is_running():
			if not self.get_server("nap").IsEnabled():
    				self.get_server("nap").Enable()
    				
    		self.dhcp_lock = False		
		
		self.connect("service-started", self.network_svc_started)
	
	def connect_signals(self):


		self.add_signal(self.on_connect, 'Connected', 'org.bluez.network.Connection', None, None, path_keyword="path")
		self.add_signal(self.on_disconnect, 'Disconnected', 'org.bluez.network.Connection', None, None, path_keyword="path")

		self.add_signal(self.on_connection_created, 'ConnectionCreated', 'org.bluez.network.Manager', None, "/org/bluez/network")
		self.add_signal(self.on_connection_removed, 'ConnectionRemoved', 'org.bluez.network.Manager', None, "/org/bluez/network")

       		self.add_signal(self.on_server_enabled, 'Enabled', 'org.bluez.network.Server', None, None, path_keyword="path")
       		self.add_signal(self.on_server_disabled, 'Disabled', 'org.bluez.network.Server', None, None, path_keyword="path")
       
    	def network_svc_started(self, obj):
    		if Globals.Settings.net_nap_autostart and not self.get_server("nap").IsEnabled():
    			self.get_server("nap").Enable()
	
	def on_server_enabled(self, path):
		self.emit("enabled", path)
    
    	def on_server_disabled(self, path):
		self.emit("disabled", path)
        
    	def on_connection_created(self, path):
		self.emit("connection-created", path)
    
    	def on_connection_removed(self, path):
		self.emit("connection-removed", path)
    
    	def on_connect(self, path):
		self.emit("connect", path)
		dhcp_clients = marshaller(os.path.expanduser("~/.blueman/dhcp.dat"))
		if path in dhcp_clients.data:
			self._set_widget_for_polkit(None)

			self.dhcp_renew(path)
		
        
    	def on_disconnect(self, path):
		self.emit("disconnect", path)
		
	#means to provide a gtk widget for polkit auth dialog, since this class is "background"
	def _set_widget_for_polkit(self, widget):
		self._polkit_widget = widget
		
	def dhcp_renew(self, path):
		
		connection = self.get_connection(path)
		
		net_interface = connection.GetInterface()
		
		p_auth = polkit_auth()
		n = notification()
		
		req_notf = None
		
		def on_dhclient_return(interface, return_code, bound_to):
			global sig_id, req_notf
			req_notf.close()
			
			self.emit("dhcp-end-request", interface, return_code, bound_to)
			
			if return_code == 0:
				n.show(_("Bound to IP address"), _("Interface %s bound to IP address %s") % (interface, bound_to), stock="bluetooth")
			else:
				n.show(_("Failed to retrieve an IP address"), _("DHCP did not provide any IP addresses on interface %s") % interface, stock="bluetooth")
				
			sig_id.remove()
			self.dhcp_lock = False
		
		
		def reply(status):
			global sig_id, req_notf
			if p_auth.is_authorized("org.blueman.dhcp.client") or status:
				self.dhcp_lock = True
				#try:
				m = mechanism()
				m.dhcpClient(net_interface)

			
				sig_id = m.connect_to_signal("dhcpReturn", on_dhclient_return)
				self.emit("dhcp-begin-request", net_interface, path)
			
				req_notf = n.show(_("Requesting an IP address"), _("Requesting an IP address on interface %s") % net_interface, stock="bluetooth", time=60000)
				#except:
				#	self.dhcp_lock = False
				
			
		def err(*args):
			print "err"
		
		
		p_auth.obtain_authorization_async(self._polkit_widget, "org.blueman.dhcp.client", reply, err)
        

    
    	def get_connections(self):
        	if self.service.IsRunning():
            		connections = self.service_manager.ListConnections()
            		ret = []
            		for i in range(len(connections)):
                		obj = self.bus.get_object(self.service_bus, connections[i])
                		conn = dbus.Interface(obj, 'org.bluez.network.Connection')
                		ret.append((conn, connections[i]))
                
            		return ret
        
        	else:
            
            		return []
    
    
	def get_connection(self, path):
		try:
            		obj = self.bus.get_object(self.service_bus, path)
            		conn = dbus.Interface(obj, 'org.bluez.network.Connection')
            		return conn
        	except:
            		return None
            		
        def find_connection(self, pattern):
		try:
			path = self.service_manager.FindConnection(pattern)
			return self.get_connection(path)
        	except:
            		return None
    
    	def get_servers(self):
        	ret = []
        	servers = self.service_manager.ListServers()
        	for i in range(len(servers)):
            		obj = self.bus.get_object(self.service_bus, servers[i])
            		serv = dbus.Interface(obj, 'org.bluez.network.Server')
            		ret.append((serv,servers[i]))
            
        	return ret
        	
        def get_server(self, pattern):
        	try:
            		path = self.service_manager.FindServer(pattern)

            		obj = self.bus.get_object(self.service_bus, path)
            		conn = dbus.Interface(obj, 'org.bluez.network.Server')
            		return conn
        	except:
            		return None
        	
        def create_connection(self, addr, uuid, **kwargs):
        	self.service_manager.CreateConnection(addr, uuid, **kwargs)
        	
        def remove_connection(self, path, **kwargs):
        	self.service_manager.RemoveConnection(path, **kwargs)
        	m=marshaller(os.path.expanduser("~/.blueman/dhcp.dat"))
        	m.data.remove(path)
        	m.save()
        	

